import git
from git import Repo
import os.path
from os import path
import os
import mysql.connector
import yaml

#connect to database and initialize cursor

mydb = mysql.connector.connect(
        host="localhost",
        user="root",
        passwd=os.environ["db_pass"],
        database="ros_ecosystem"
)
mycursor = mydb.cursor(buffered=True)

good_commits = []

def add_to_db(name, type, url, version, commit):
    if is_unique(name, type, version):
        datetime = get_date(commit)
        add_repo = ("INSERT INTO all_versions "
                    "(name, distro, type, repo_url, version, release_datetime, rosdistro_commit ) "
                    "VALUES (%s, %s, %s, %s, %s, %s, %s)")
        data_repo = (name, "kinetic", type, url, version, datetime, commit)
        mycursor.execute(add_repo, data_repo)
       mydb.commit()

def get_date(commit):
    get_date = 'SELECT datetime FROM good_commits WHERE commit="'+commit+'"'
    mycursor.execute(get_date)
    the_date = mycursor.fetchone()
    date = the_date[0]
    datetime = date.strftime("%Y-%m-%d %H:%M:%S")
    return datetime

def is_unique(name, type, version):
    check_exist = 'SELECT id FROM all_versions WHERE name="'+name+'" AND type="'+type+'" AND version="'+version+'"'
    mycursor.execute(check_exist)
    result = mycursor.fetchone()
    if result is None:
        return True
    else:
        return False

def add_to_bad(name, commit):
    add_repo = ("INSERT INTO bad_repos "
                "(name, commit)"
                "VALUES (%s, %s)")
    data = (name, commit)
    mycursor.execute(add_repo, data)

def add_sub_pack(name, type, url, version, commit, parent):
    is_sub_pack = "1"
    if is_unique(name, type, version):
    #get parent id 
        get_parent_id = 'SELECT id FROM all_versions WHERE name="'+parent+'" AND version="'+version+'" AND type="'+type+'"'    
        mycursor.execute(get_parent_id)
        parent_id_tup = mycursor.fetchone()
        parent_id = parent_id_tup[0]
        datetime = get_date(commit)
        add_repo = ("INSERT INTO all_versions "
                    "(name, distro, version, type, repo_url, is_sub_pack, release_datetime, rosdistro_commit, parent_id ) "
                     "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)")
        data_repo = (name, "kinetic", version, type, url, is_sub_pack, datetime, commit, parent_id)
        mycursor.execute(add_repo, data_repo)
        mydb.commit()


#see if a file changed between two commits
    
def change_between(old_commit, new_commit, file):
    old_commit_str = str(old_commit)
    new_commit_str = str(new_commit)
    myCmd = 'git diff --name-only ' + old_commit_str +" "+ new_commit_str + " > out.txt"
    os.system(myCmd)
    f = open('out.txt', "r")
    for line in f:
        temp = line.replace('\n', '')
        if temp == file:
            f.close()
            os.remove('out.txt')
            return True
    f.close()
    os.remove('out.txt')
    return False
   
#remove commits where the yaml file did not exist

def filter_commits(repo, all_commits):
    yaml_path = "/home/sophia/ros-ecosystem-project/temp/rosdistro/kinetic/distribution.yaml"
    for commit in all_commits:
        checkout_commit(repo, commit)
        if os.path.exists(yaml_path):
            good_commits.append(commit)
        else:
            continue
    print("good commits length", len(good_commits))
#will have access to the commit/date and eventually pass it to the database
#making method
def checkout_commits(repo, safe_commits):
    os.chdir('temp/rosdistro')
    for commit in safe_commits:
        repo.git.checkout(commit)
        print("checking out")
        commit_str = ''.join(commit)
        parse_yaml('kinetic/distribution.yaml', commit_str)

def checkout_commit(repo, commit):
    repo.git.checkout(commit)

def yaml_loader(filepath):
    with open(filepath, "r") as file_descriptor:
        data = yaml.load(file_descriptor)
        return data
        
def parse_yaml(filepath, commit):
    data = yaml_loader(filepath)
    for k, v in data.items():
        if isinstance(v, dict):
            for repo, info in v.items():
                if isinstance(info, dict):
                    pass_on(repo, info, commit)
                    
def pass_on(repo, info, commit):
    found = False
    for rel_src_doc, more_info in info.items():
        is_stack = "0"
        if rel_src_doc == "release" or rel_src_doc == "source":
            found = True
            if isinstance(more_info, dict):
                for url_ver_pack, value in more_info.items():
                    if url_ver_pack == 'url': 
                        url = more_info[url_ver_pack]
                    if url_ver_pack == 'version':
                        version = more_info[url_ver_pack]
                        add_to_db(repo, rel_src_doc, url, version, commit)
                        for url_ver_pack, value in more_info.items():
                            if url_ver_pack == 'packages':
                                is_stack = "1"
                                for subpack in more_info[url_ver_pack]:
                                    add_sub_pack(subpack, rel_src_doc, url, version, commit, repo)
         
    if not found:
        add_to_bad(repo, commit)
            
'''adds commits where yaml file was changed **and their dates** to database
so that the slow code only has to be run once'''

def make_commit_db(commit_set):
    for commit in commit_set:
        commit_str = str(commit)
        myCmd = "git show -s --format=%ci " + commit_str + " > date.txt"
        os.system(myCmd)
        f = open('date.txt', "r")
        full_date = f.readline()
        date_parts = full_date.split()
        date = date_parts[0] + " " + date_parts[1]
        add_commit = ("INSERT INTO good_commits "
                   "(commit, datetime) "
                   "VALUES (%s, %s)")
        data_commit = (commit_str, date)
        mycursor.execute(add_commit, data_commit)
        mydb.commit()
        f.close()
        os.remove('date.txt')

'''from first-->last commit, pass in the sha for each consecutive commit
to see if the given file changed'''

def check_for_change(good_commits):
    os.chdir('temp/rosdistro')
    good_commits.reverse()
    to_checkout = []
    for i in range(len(good_commits)):
        if i != len(good_commits)-1:
            if change_between(good_commits[i], good_commits[i+1], "kinetic/distribution.yaml"):
                to_checkout.append(good_commits[i])
                to_checkout.append(good_commits[i+1])
            else:
                continue
    good_commits.reverse()
    return list(set(to_checkout))

        
def main():
    print("starting")

    url = "https://github.com/ros/rosdistro"
    path = "/home/sophia/ros-ecosystem-project/temp"
    #git.Git(path).clone(url)
    full_path = path + "/rosdistro"
    repo = Repo(full_path)
    assert not repo.bare
    all_commits = list(repo.iter_commits('master'))
    print("all commits length", len(all_commits))
    yaml_path = "/home/sophia/ros-ecosystem-project/temp/rosdistro/kinetic/distribution.yaml"
    #filter_commits(repo, all_commits)
    #commit_set = check_for_change(good_commits)
    #make_commit_db(commit_set)
    command = "SELECT commit FROM good_commits"
    mycursor.execute(command)
    safe_commits = mycursor.fetchall()
    checkout_commits(repo, safe_commits)
    
    print("done")

if __name__ == '__main__': main()
